/**
 * HlClock- SourceMod plugin announces the current server time and round
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#pragma semicolon 1
#include <sourcemod>
#include <sdktools>

#define PLUGIN_VERSION "1.0.3.2"

public Plugin:myinfo =
{
	name = "HlClock",
	author = "Blade",
	description = "Announces the current server time and round",
	version = PLUGIN_VERSION,
	url = "http://www.germanskifferstuebchen.de"
};

new String:downloads[PLATFORM_MAX_PATH];
new String:g_hour[MAXPLAYERS + 1][24];
new String:g_minute[MAXPLAYERS + 1][24];
new String:g_second[MAXPLAYERS + 1][24];

new Handle:g_hTimerList[MAXPLAYERS + 1];
new Handle:g_Cvar_MsgMode = INVALID_HANDLE;

new RoundCount = 1;

public OnPluginStart()
{
	CreateConVar("hlclock_version", PLUGIN_VERSION, "Half Life Clock", FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY);
	g_Cvar_MsgMode = CreateConVar("sm_time_display_mode", "1", "1 = Top left. 0 = Chat area", 0, true, 0.0, true, 1.0);

	RegConsoleCmd("say", OnPlayerSay);
	HookEvent("round_end", EventRoundEnd);
}

public OnMapStart()
{
	CreateTimer(0.1, Parse_File);
	RoundCount = 1;
}

public Action:Parse_File(Handle:timer)
{
	BuildPath(Path_SM,downloads,sizeof(downloads),"configs/hlclock.txt");
	if(!FileExists(downloads)) {
		LogMessage("hlclock.txt not parsed...file doesn't exist!");
	}else{
		new Handle:soundhandle = OpenFile(downloads, "r");

		new String:g_soundfile[66];
		while(!IsEndOfFile(soundhandle)){
			ReadFileLine(soundhandle, g_soundfile, sizeof(g_soundfile));
			TrimString(g_soundfile);

			// add to downloadables
			AddFileToDownloadsTable(g_soundfile);

			// precache sound
			ReplaceString(g_soundfile, sizeof(g_soundfile), "sound/", "");
			PrecacheSound(g_soundfile, true);
		}
		CloseHandle(soundhandle);
	}
}

public EventRoundEnd(Handle:event, const String:name[], bool:dontBroadcast)
{
	new winner = GetEventInt(event, "winner");
	new reason = GetEventInt(event, "reason");

	// Make sure that we have a normal round end
	if(reason == 16)
	{
		return;
	}

	// Make sure that the winner is not something strange
	if(winner != 2 && winner != 3)
	{
		return;
	}

	RoundCount++;
}

public Action:OnPlayerSay(client, args)
{
	decl String:text[192];
	GetCmdArgString(text, sizeof(text));

	new startidx;
	new len = strlen(text);
	if (text[len-1] == '"')
	{
		text[len-1] = '\0';
		startidx = 1;
	}

	if (strcmp(text[startidx], "time", false) == 0){

		decl String:time[64];
		FormatTime(time, 64, "%H:%M:%S");

		new String:split[3][10];
		ExplodeString(time, ":",split,3,10);

		decl String:sBuffer[92];
		Format(sBuffer, 92, "Thetime is: %s:%s:%s",split[0], split[1], split[2]);

		// display the time
		if(GetConVarBool(g_Cvar_MsgMode))
			DisplayTime(sBuffer);
		else
			PrintToChatAll(sBuffer);

		EmitSoundToClient(client, "hlclock/thetime.wav");

		g_hour[client] = split[0];
		g_minute[client] = split[1];
		g_second[client] = split[2];

		g_hTimerList[client] = CreateTimer(1.5, Announce_Time, client);
	} else if (strcmp(text[startidx], "round", false) == 0){
		EmitSoundToClient(client, "hlclock/round.wav");
		g_hTimerList[client] = CreateTimer(0.7, Announce_Round, client);
	}
	return Plugin_Continue;
}

public Action:Announce_Round(Handle:timer, any:client)
{
	if(!IsClientInGame(client) || !IsClientConnected(client)){
		return Plugin_Handled;
	}

	decl String:sBuffer[64];
	Format(sBuffer, sizeof(sBuffer), "hlclock/%i.wav", RoundCount);
	EmitSoundToClient(client, sBuffer);

	return Plugin_Handled;
}

public Action:Announce_Time(Handle:timer, any:client)
{
	if(!IsClientInGame(client) || !IsClientConnected(client)){
		return Plugin_Handled;
	}

	decl String:sBuffer[100];
	if(g_hour[client][0] != '\0'){

		Format(sBuffer, sizeof(sBuffer), "hlclock/%s.wav", g_hour[client]);
		EmitSoundToClient(client, sBuffer);

		g_hour[client][0] = '\0';
		g_hTimerList[client] = CreateTimer(1.1, Announce_Time, client);
	} else {
		Format(sBuffer, sizeof(sBuffer), "hlclock/%s.wav", g_minute[client]);
		EmitSoundToClient(client, sBuffer);

		g_hTimerList[client] = INVALID_HANDLE;
	}

	return Plugin_Handled;
}

DisplayTime(const String:time[])
{
	new Handle:top_values = CreateKeyValues("msg");
	KvSetString(top_values, "title", time);
	KvSetNum(top_values, "level", 1);
	KvSetString(top_values, "time", "7");

	for (new i = 1; i <= GetMaxClients(); i++)
	{
		if (IsClientInGame(i))
		{
			CreateDialog(i, top_values, DialogType_Msg);
		}
	}
	CloseHandle(top_values);
}